package com.energy.manage.service.controller.company;


import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.service.config.annotations.UserLoginToken;
import com.energy.manage.service.domain.dto.company.*;
import com.energy.manage.service.domain.vo.company.WindCompanyPageVo;
import com.energy.manage.service.domain.vo.company.WindCompanyVo;
import com.energy.manage.service.domain.vo.windfield.WindFieldVo;
import com.energy.manage.service.service.company.WindCompanyService;
import com.energy.manage.service.service.windfield.WindFieldService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@CrossOrigin
@Api(value = "WindCompanyController", tags = "企业管理接口")
@RequestMapping(value = "/api/wind/company")
public class WindCompanyController {

    @Autowired
    private WindCompanyService windCompanyService;

    @Autowired
    private WindFieldService windFieldService;


    @UserLoginToken
    @PostMapping(value = "/addCompany")
    @ApiOperation(value = "创建企业")
    public ResultResp addCompany(@RequestBody WindCompanyDto dto) {
        boolean flg = windCompanyService.addCompany(dto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }


    @UserLoginToken
    @PostMapping(value = "/delCompany")
    @ApiOperation(value = "删除企业")
    public ResultResp delCompany(@RequestBody WindCompanyDelDto dto) {

        List<WindFieldVo> list = windFieldService.getWindFieldVos(dto.getCompanyCode());
        if (list.size() > 0) {
            return ResultResp.FAIL("项目公司有绑定风场不可删除！");
        }
        boolean flg = windCompanyService.delCompany(dto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }

    @UserLoginToken
    @PostMapping(value = "/updateStateCompany")
    @ApiOperation(value = "更新企业状态")
    public ResultResp updateStateCompany(@RequestBody WindCompanyStateDto dto) {
        boolean flg = windCompanyService.updateStateCompany(dto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }

    @UserLoginToken
    @PostMapping(value = "/getWindCompany")
    @ApiOperation(value = "查询企业对象")
    public ResultResp<WindCompanyVo> getWindCompany(@RequestBody WindCompanyDelDto dto) {
        WindCompanyVo vo = windCompanyService.getWindCompany(dto);
        return ResultResp.SUCCESS(vo) ;
    }

    @UserLoginToken
    @PostMapping(value = "/updateCompany")
    @ApiOperation(value = "更新企业")
    public ResultResp updateCompany(@RequestBody WindCompanyUpdateDto dto) {
        boolean flg = windCompanyService.updateCompany(dto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }


    @UserLoginToken
    @PostMapping(value = "/getAllWindCompany")
    @ApiOperation(value = "查询企业树")
    public ResultResp<List<Map<String, Object>>> getAllWindCompany(@RequestBody WindCompanyPageDto windCompanyPageDto) {

        List<Map<String, Object>> list = windCompanyService.getAllWindCompany(windCompanyPageDto.getState(), windCompanyPageDto.getCompanyName());
        return ResultResp.SUCCESS(list);
    }




}
