package com.energy.manage.service.controller.windfield;


import com.energy.manage.common.base.Page;
import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.service.config.annotations.UserLoginToken;
import com.energy.manage.service.domain.dto.windfield.*;
import com.energy.manage.service.domain.vo.company.WindCompanyPageVo;
import com.energy.manage.service.domain.vo.windfield.WindFieldPageVo;
import com.energy.manage.service.domain.vo.windfield.WindFieldVo;
import com.energy.manage.service.domain.vo.windrelation.WindRelationVo;
import com.energy.manage.service.service.windfield.WindFieldResourceService;
import com.energy.manage.service.service.windfield.WindFieldService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@CrossOrigin
@Api(value = "WindFieldController", tags = "风场管理接口")
@RequestMapping(value = "/api/wind/field")
public class WindFieldController {

    @Autowired
    private WindFieldService windFieldService;

    @Autowired
    private WindFieldResourceService windFieldResourceService;


    @UserLoginToken
    @PostMapping(value = "/createWindField")
    @ApiOperation(value = "创建风场信息")
    public ResultResp createWindField(@RequestBody WindFieldDto windFieldDto) {
        boolean flg = windFieldService.createWindField(windFieldDto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }


    @UserLoginToken
    @PostMapping(value = "/windFieldImportData")
    @ApiOperation(value = "导入风场信息")
    public ResultResp windFieldImportData(@RequestPart MultipartFile file,
                                          @RequestParam(required = true) String companyCode,
                                          @RequestParam(required = true) Integer provinceId,
                                          @RequestParam(required = true) String provinceName,
                                          @RequestParam(required = true) Integer cityId,
                                          @RequestParam(required = true) String cityName,
                                          @RequestParam(required = true) String areaCode) {

        WindFieldImportDataDto windFieldImportDataDto = new WindFieldImportDataDto();
        windFieldImportDataDto.setCompanyCode(companyCode);
        windFieldImportDataDto.setProvinceId(provinceId);
        windFieldImportDataDto.setProvinceName(provinceName);
        windFieldImportDataDto.setCityId(cityId);
        windFieldImportDataDto.setCityName(cityName);
        windFieldImportDataDto.setAreaCode(areaCode);

        boolean flg = windFieldService.windFieldImportData(file, windFieldImportDataDto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }

    @Deprecated
    @UserLoginToken
    @PostMapping(value = "/afreshWindFieldImportData")
    @ApiOperation(value = "重新导入风场信息")
    public ResultResp afreshWindFieldImportData(@RequestPart MultipartFile file,
                                                @RequestParam(required = true) String companyCode,
                                                @RequestParam(required = true) Integer provinceId,
                                                @RequestParam(required = true) String provinceName,
                                                @RequestParam(required = true) Integer cityId,
                                                @RequestParam(required = true) String cityName,
                                                @RequestParam(required = true) String areaCode) {

//        WindFieldImportDataDto windFieldImportDataDto = new WindFieldImportDataDto();
//        windFieldImportDataDto.setCompanyCode(companyCode);
//        windFieldImportDataDto.setProvinceId(provinceId);
//        windFieldImportDataDto.setProvinceName(provinceName);
//        windFieldImportDataDto.setCityId(cityId);
//        windFieldImportDataDto.setCityName(cityName);
//        windFieldImportDataDto.setAreaCode(areaCode);
//
//        boolean flg = windFieldService.afreshWindFieldImportData(file, windFieldImportDataDto);
//        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
        return null;
    }

    @UserLoginToken
    @PostMapping(value = "/WindFieldListPage")
    @ApiOperation(value = "分页查询风场信息")
    public ResultResp<Page<WindFieldPageVo>> WindFieldListPage(@RequestBody WindFieldPageDto windFieldPageDto) {
        Page<WindFieldPageVo> page = windFieldService.WindFieldListPage(windFieldPageDto);
        return ResultResp.SUCCESS(page);
    }


    @UserLoginToken
    @PostMapping(value = "/delWindFieldById")
    @ApiOperation(value = "删除风场")
    public ResultResp delWindFieldById(@RequestBody WindFieldCodeDto windFieldCodeDto) {

        boolean flg = windFieldService.delWindFieldById(windFieldCodeDto.getFieldCode());
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }

    @UserLoginToken
    @PostMapping(value = "/getWindFieldVos")
    @ApiOperation(value = "通过企业编号查询关联风场")
    public ResultResp<List<WindFieldVo>> getWindFieldVos(@RequestParam(required = true) String companyCode) {

        List<WindFieldVo>  list = windFieldService.getWindFieldVos(companyCode);
        return ResultResp.SUCCESS(list);
    }

    @UserLoginToken
    @PostMapping(value = "/updateWindFieldStateById")
    @ApiOperation(value = "修改状态")
    public ResultResp updateWindFieldStateById(@RequestBody WindFieldUpdateStateDto windFieldUpdateStateDto) {

        boolean flg = windFieldService.updateWindFieldStateById(windFieldUpdateStateDto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }

    @UserLoginToken
    @PostMapping(value = "/getWindFieldVo")
    @ApiOperation(value = "查询风场对象")
    public ResultResp<WindFieldVo> getWindFieldVo(@RequestBody WindFieldCodeDto windFieldCodeDto) {
        WindFieldVo vo = windFieldService.getWindFieldVo(windFieldCodeDto);
        return ResultResp.SUCCESS(vo);
    }

    @UserLoginToken
    @PostMapping(value = "/WindFieldDto")
    @ApiOperation(value = "更新风场信息")
    public ResultResp updateWindField(@RequestBody WindFieldUpdateDto windFieldUpdateDto) {
        boolean flg = windFieldService.updateWindField(windFieldUpdateDto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }

    @UserLoginToken
    @PostMapping(value = "/getWindFieldTree")
    @ApiOperation(value = "查询风场树")
    public ResultResp< List<Map<String, Object>>> getWindFieldTree() {
        List<Map<String, Object>> list = windFieldService.getWindFieldTree();
        return ResultResp.SUCCESS(list);
    }


    @UserLoginToken
    @PostMapping(value = "/saveWindFieldResource")
    @ApiOperation(value = "保存上传风场资源文件")
    public ResultResp saveWindFieldResource(@RequestBody WindFieldResourceCreateDto windFieldResourceCreateDto) {
        boolean flg = windFieldResourceService.saveWindFieldResource(windFieldResourceCreateDto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }


    @UserLoginToken
    @PostMapping(value = "/delWindFieldResource")
    @ApiOperation(value = "删除风场资源文件")
    public ResultResp delWindFieldResource(@RequestBody WindFieldResourceDelDto windFieldResourceDelDto) {
        boolean flg = windFieldResourceService.delWindFieldResource(windFieldResourceDelDto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }








}
