package com.energy.manage.service.service.windfield.impl;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import com.alibaba.excel.EasyExcel;
import com.energy.manage.common.base.BaseServiceImpl;
import com.energy.manage.common.base.Page;
import com.energy.manage.common.constant.Constants;
import com.energy.manage.common.constant.redis.ManagerRedisKeyConstant;
import com.energy.manage.common.enums.DeleteStatusEnum;
import com.energy.manage.common.enums.IdPrefixEnum;
import com.energy.manage.common.enums.TypeRelationEnum;
import com.energy.manage.common.po.company.WindCompanyPO;
import com.energy.manage.common.po.windfield.WindFieldPO;
import com.energy.manage.common.util.IdGeneratorUtil;
import com.energy.manage.service.domain.dto.anemometertower.AnemometerTowerDto;
import com.energy.manage.service.domain.dto.anemometertower.AnemometerTowerRelationDto;
import com.energy.manage.service.domain.dto.company.WindCompanyDto;
import com.energy.manage.service.domain.dto.windfield.*;
import com.energy.manage.service.domain.dto.windrelation.WindRelationDto;
import com.energy.manage.service.domain.vo.anemometertower.AnemometerTowerByFieldVo;
import com.energy.manage.service.domain.vo.excel.WindFieldExceVo;
import com.energy.manage.service.domain.vo.windfield.WindFieldPageVo;
import com.energy.manage.service.domain.vo.windfield.WindFieldResourceVo;
import com.energy.manage.service.domain.vo.windfield.WindFieldVo;
import com.energy.manage.service.domain.vo.windrelation.WindRelationVo;
import com.energy.manage.service.mappers.windfield.WindFieldMapper;
import com.energy.manage.service.service.anemometertower.AnemometerTowerService;
import com.energy.manage.service.service.cache.CacheService;
import com.energy.manage.service.service.windfield.WindFieldResourceService;
import com.energy.manage.service.service.windfield.WindFieldService;
import com.energy.manage.service.service.windrelation.WindRelationService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import java.util.Date;
import java.util.List;
import java.util.Map;


@Slf4j
@Service
public class WindFieldServiceImpl extends BaseServiceImpl<WindFieldPO> implements WindFieldService {


    @Autowired
    private CacheService cacheService;


    @Autowired
    private WindFieldMapper windFieldMapper;

    @Autowired
    private WindRelationService windRelationServic;

    @Autowired
    private AnemometerTowerService anemometerTowerService;

    @Autowired
    private WindFieldResourceService windFieldResourceService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean createWindField(WindFieldDto windFieldDto) {

        WindFieldPO windFieldPO = new WindFieldPO();
        BeanUtil.copyProperties(windFieldDto, windFieldPO);
        String number = IdPrefixEnum.WIND_FIELD_NUMBER.getCode().concat(windFieldDto.getAreaCode() + IdGeneratorUtil.zeroFillUtil(cacheService.incr(ManagerRedisKeyConstant.build(ManagerRedisKeyConstant.IDGENERATOR_CONSTANTS_KEY, IdPrefixEnum.WIND_FIELD_NUMBER.getCode()))));
        windFieldPO.setFieldCode(number);
        windFieldPO.setCreateTime(new Date());
        windFieldPO.setUpdateTime(new Date());
        windFieldPO.setState(Constants.GLOBAL_IN_USE);
        windFieldPO.setDelState(DeleteStatusEnum.NODELETE.getCode());
        if (windFieldMapper.insertUseGeneratedKeys(windFieldPO) <= 0) {
            return false;
        }

        // 处理测风塔数据
        if (!CollectionUtils.isEmpty(windFieldDto.getAnemometerTowerRelationDtos())) {
            AnemometerTowerRelationDto anemometerTowerRelationDto = null;
            List<AnemometerTowerRelationDto> anemometerTowerDtos = Lists.newArrayList();
            for(WindFieldDto.AnemometerTowerRelationDto anemometerTowerRelationDto1 : windFieldDto.getAnemometerTowerRelationDtos()){
                anemometerTowerRelationDto = new AnemometerTowerRelationDto();
                BeanUtils.copyProperties(anemometerTowerRelationDto1, anemometerTowerRelationDto);
                anemometerTowerRelationDto.setFieldCode(windFieldDto.getFieldCode());
                anemometerTowerDtos.add(anemometerTowerRelationDto);
            }
            anemometerTowerService.bathAnemometerTowerRelation(anemometerTowerDtos);
        }
        return createWindRelation(windFieldPO);
    }


    /**
     * 创建关系数据
     *
     * @param po
     * @return
     */
    private boolean createWindRelation(WindFieldPO po) {

        WindRelationDto windRelationDto = new WindRelationDto();
        windRelationDto.setCodeNumber(po.getFieldCode());
        windRelationDto.setCodeName(po.getFieldName());
        windRelationDto.setParentCode(po.getCompanyCode());
        windRelationDto.setType(TypeRelationEnum.WIND_FIELD_NUMBER.getCode());

        return windRelationServic.createWindRelation(windRelationDto);
    }

    @SneakyThrows
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean windFieldImportData(MultipartFile file, WindFieldImportDataDto windFieldImportDataDto) {
        List<WindFieldExceVo> windFieldExceVos = EasyExcel.read(file.getInputStream()).head(WindFieldExceVo.class).sheet().doReadSync();
        if (CollectionUtils.isEmpty(windFieldExceVos)) {
            return false;
        }
        WindFieldPO windFieldPO = null;
        List<WindFieldPO> list = Lists.newArrayList();

        for (WindFieldExceVo windFieldExceVo : windFieldExceVos) {
            windFieldPO = this.createWindFieldPO(windFieldImportDataDto);
            windFieldPO.setFieldName(windFieldExceVo.getFieldName());
            windFieldPO.setDensity(windFieldExceVo.getDensity());
            windFieldPO.setLatitude(windFieldExceVo.getLatitude());
            windFieldPO.setLongitude(windFieldExceVo.getLongitude());
            windFieldPO.setElevationHeight(windFieldExceVo.getElevationHeight());
            list.add(windFieldPO);
        }
        if (CollectionUtils.isEmpty(list)) {
            return false;
        }
        if (windFieldMapper.insertList(list) <= 0) {
            return false;
        }

        return bathCreateWindRelation(list);
    }

    /**
     * 批量创建关系数据
     *
     * @param list
     * @return
     */
    private boolean bathCreateWindRelation(List<WindFieldPO> list) {

        WindRelationDto windRelationDto = null;
        List<WindRelationDto> windRelationDtos = Lists.newArrayList();
        for (WindFieldPO po : list) {
            windRelationDto = new WindRelationDto();
            windRelationDto.setCodeNumber(po.getFieldCode());
            windRelationDto.setCodeName(po.getFieldName());
            windRelationDto.setParentCode(po.getCompanyCode());
            windRelationDto.setType(TypeRelationEnum.WIND_FIELD_NUMBER.getCode());
            windRelationDtos.add(windRelationDto);
        }
        return windRelationServic.createWindRelationList(windRelationDtos);
    }


    @Override
    public boolean afreshWindFieldImportData(MultipartFile file, WindFieldImportDataDto windFieldImportDataDto) {
        // 删除企业下所有风场
        if (!delWindFieldByCompanyCode(windFieldImportDataDto.getCompanyCode())) {
            return false;
        }
        return windFieldImportData(file, windFieldImportDataDto);
    }

    /**
     * 创建对象
     *
     * @param windFieldImportDataDto
     * @return
     */
    private WindFieldPO createWindFieldPO(WindFieldImportDataDto windFieldImportDataDto) {
        WindFieldPO windFieldPO = new WindFieldPO();
        BeanUtil.copyProperties(windFieldImportDataDto, windFieldPO);
        String number = IdPrefixEnum.WIND_FIELD_NUMBER.getCode().concat(windFieldImportDataDto.getAreaCode() + IdGeneratorUtil.zeroFillUtil(cacheService.incr(ManagerRedisKeyConstant.build(ManagerRedisKeyConstant.IDGENERATOR_CONSTANTS_KEY, IdPrefixEnum.WIND_FIELD_NUMBER.getCode()))));
        windFieldPO.setFieldCode(number);
        windFieldPO.setCreateTime(new Date());
        windFieldPO.setUpdateTime(new Date());
        windFieldPO.setState(Constants.GLOBAL_IN_USE);
        windFieldPO.setDelState(DeleteStatusEnum.NODELETE.getCode());
        return windFieldPO;
    }


    @Override
    public Page<WindFieldPageVo> WindFieldListPage(WindFieldPageDto windFieldPageDto) {
        PageHelper.startPage(windFieldPageDto.getPageNum(), windFieldPageDto.getPageSize());
        List<WindFieldPageVo> list = windFieldMapper.selectWindFieldPageByfield(windFieldPageDto);
        Long total = org.apache.commons.collections.CollectionUtils.isEmpty(list) ? 0L : new PageInfo<>(list).getTotal();
        return Page.build(windFieldPageDto.getPageNum(), windFieldPageDto.getPageSize(), total, list);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean delWindFieldByCompanyCode(String companyCode) {
        WindFieldPO windFieldPO = new WindFieldPO();
        windFieldPO.setDelState(DeleteStatusEnum.DELETE.getCode());
        Example queryExample = new Example(WindFieldPO.class);
        Example.Criteria criteria = queryExample.createCriteria();
        criteria.andEqualTo("companyCode", companyCode);
        return windFieldMapper.updateByExampleSelective(windFieldPO, queryExample) > 0;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean delWindFieldById(String fieldCode) {
        WindFieldPO windFieldPO = new WindFieldPO();
        windFieldPO.setDelState(DeleteStatusEnum.DELETE.getCode());
        Example queryExample = new Example(WindFieldPO.class);
        Example.Criteria criteria = queryExample.createCriteria();
        criteria.andEqualTo("fieldCode", fieldCode);
        return windFieldMapper.updateByExampleSelective(windFieldPO, queryExample) > 0;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateWindFieldStateById(WindFieldUpdateStateDto windFieldUpdateStateDto) {
        WindFieldPO windFieldPO = new WindFieldPO();
        windFieldPO.setState(windFieldUpdateStateDto.getState());
        Example queryExample = new Example(WindFieldPO.class);
        Example.Criteria criteria = queryExample.createCriteria();
        criteria.andEqualTo("fieldCode", windFieldUpdateStateDto.getFieldCode());
        return windFieldMapper.updateByExampleSelective(windFieldPO, queryExample) > 0;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateWindFieldNumber(String fieldCode) {
        return windFieldMapper.updateWindFieldEngineNumberByFieldCode(fieldCode) > 0;
    }

    @Override
    public WindFieldVo getWindFieldVo(WindFieldCodeDto windFieldCodeDto) {
        WindFieldPO windFieldPO = new WindFieldPO();
        windFieldPO.setFieldCode(windFieldCodeDto.getFieldCode());
        WindFieldPO po = windFieldMapper.selectOne(windFieldPO);
        if (po == null) {
            return null;
        }
        WindFieldVo windFieldVo = new WindFieldVo();
        BeanUtil.copyProperties(po, windFieldVo);

        //测风塔数据
        List<AnemometerTowerByFieldVo> list = anemometerTowerService.getAnemometerTowerByField(windFieldCodeDto.getFieldCode());
        if (!CollectionUtils.isEmpty(list)) {
            windFieldVo.setAnemometerTowerByFieldVos(list);
        }

        //查询风场对应资源信息
        List<WindFieldResourceVo> windFieldResourceVos = windFieldResourceService.getWindFieldResourceByFieldCode(windFieldCodeDto.getFieldCode());
        if (!CollectionUtils.isEmpty(windFieldResourceVos)) {
            windFieldVo.setWindFieldResourceVos(windFieldResourceVos);
        }

        return windFieldVo;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateWindField(WindFieldUpdateDto windFieldUpdateDto) {

        WindFieldPO windFieldPO = new WindFieldPO();
        BeanUtil.copyProperties(windFieldUpdateDto, windFieldPO);
        Example queryExample = new Example(WindFieldPO.class);
        Example.Criteria criteria = queryExample.createCriteria();
        criteria.andEqualTo("fieldCode", windFieldPO.getFieldCode());

        boolean flg = true;
        if (windFieldMapper.updateByExampleSelective(windFieldPO, queryExample) <= 0) {
            return false;
        }

        if(!StringUtils.isEmpty(windFieldUpdateDto.getFieldCode())&&!StringUtils.isEmpty(windFieldUpdateDto.getFieldName())){
            WindRelationDto windRelationDto = new WindRelationDto();
            windRelationDto.setCodeName(windFieldUpdateDto.getFieldName());
            windRelationDto.setCodeNumber(windFieldUpdateDto.getFieldCode());
            flg =  windRelationServic.updateWindRelationName(windRelationDto);
            if(!flg){
                log.warn("修改风场关系树表失败！");
            }
        }

        // 处理测风塔数据
        if (!CollectionUtils.isEmpty(windFieldUpdateDto.getAnemometerTowerRelationDtos())) {
            anemometerTowerService.deltAnemometerTowerRelationByField(windFieldUpdateDto.getFieldCode());
            // 处理测风塔数据
            if (!CollectionUtils.isEmpty(windFieldUpdateDto.getAnemometerTowerRelationDtos())) {
                AnemometerTowerRelationDto anemometerTowerRelationDto = null;
                List<AnemometerTowerRelationDto> anemometerTowerDtos = Lists.newArrayList();
                for(WindFieldUpdateDto.AnemometerTowerRelationDto anemometerTowerRelationDto1 : windFieldUpdateDto.getAnemometerTowerRelationDtos()){
                    anemometerTowerRelationDto = new AnemometerTowerRelationDto();
                    BeanUtils.copyProperties(anemometerTowerRelationDto1, anemometerTowerRelationDto);
                    anemometerTowerRelationDto.setFieldCode(windFieldUpdateDto.getFieldCode());
                    anemometerTowerDtos.add(anemometerTowerRelationDto);
                }
                flg = anemometerTowerService.bathAnemometerTowerRelation(anemometerTowerDtos);
                if(!flg){
                    log.warn("修改测风塔关系表失败！");
                }
            }
        }

        return true;
    }

    @Override
    public List<WindFieldVo> getWindFieldVos(String companyCode) {
        WindFieldPO windFieldPO = new WindFieldPO();
        windFieldPO.setCompanyCode(companyCode);
        List<WindFieldPO> list = windFieldMapper.select(windFieldPO);

        WindFieldVo windFieldVo = null;
        List<WindFieldVo> windFieldVos = Lists.newArrayList();
        for(WindFieldPO windFieldPO1 : list){
            windFieldVo = new WindFieldVo();
            BeanUtil.copyProperties(windFieldPO1,windFieldVo);
            windFieldVos.add(windFieldVo);
        }
        return windFieldVos;
    }

    @Override
    public List<Map<String, Object>> getWindFieldTree() {
        return windRelationServic.getWindRelationTree();
    }


}
